import sys

import click

from twyn.__version__ import __version__
from twyn.base.constants import (
    DEPENDENCY_FILE_MAPPING,
    SELECTOR_METHOD_MAPPING,
    AvailableLoggingLevels,
)
from twyn.core.config_handler import ConfigHandler
from twyn.main import check_dependencies


@click.group()
@click.version_option(__version__, "--version")
def entry_point() -> None:
    pass


@entry_point.command()
@click.option("--config", type=click.STRING)
@click.option(
    "--dependency-file",
    type=click.Choice(list(DEPENDENCY_FILE_MAPPING.keys())),
    help=(
        "Dependency file to analyze. By default, twyn will search in the current directory "
        "for supported files, but this option will override that behavior."
    ),
)
@click.option(
    "--selector-method",
    type=click.Choice(list(SELECTOR_METHOD_MAPPING.keys())),
    help=(
        "Which method twyn should use to select possible typosquats. "
        "`first-letter` only compares dependencies that share the first letter, "
        "while `nearby-letter` compares against dependencies whose first letter "
        "is nearby in an English keyboard. `all` compares the given dependencies "
        "against all of those in the reference."
    ),
)
@click.option(
    "-v",
    default=False,
    is_flag=True,
)
@click.option(
    "-vv",
    default=False,
    is_flag=True,
)
def run(
    config: str,
    dependency_file: str,
    selector_method: str,
    v: bool,
    vv: bool,
) -> int:
    if v and vv:
        raise ValueError(
            "Only one verbosity level is allowed. Choose either -v or -vv."
        )

    if v:
        verbosity = AvailableLoggingLevels.info
    elif vv:
        verbosity = AvailableLoggingLevels.debug
    else:
        verbosity = AvailableLoggingLevels.none

    return int(
        check_dependencies(
            config_file=config,
            dependency_file=dependency_file,
            selector_method=selector_method,
            verbosity=verbosity,
        )
    )


@entry_point.group()
def allowlist() -> None:
    pass


@allowlist.command()
@click.argument("package_name")
def add(package_name: str) -> None:
    ConfigHandler().add_package_to_allowlist(package_name)


@allowlist.command()
@click.argument("package_name")
def remove(package_name: str) -> None:
    ConfigHandler().remove_package_from_allowlist(package_name)


if __name__ == "__main__":
    sys.exit(entry_point())



from twyn.base.exceptions import TwynError


class InvalidJSONError(TwynError):
    message = "Could not json decode the downloaded packages list"


class InvalidPyPiFormatError(TwynError, KeyError):
    message = "Invalid JSON format."


class EmptyPackagesListError(TwynError):
    message = "Downloaded packages list is empty"


class CharacterNotInMatrixError(TwynError, KeyError): ...



import logging
from os import getcwd
from pathlib import Path
from typing import Any, Optional

from tomlkit import dumps, parse

from twyn.base.constants import DEFAULT_PROJECT_TOML_FILE
from twyn.core.exceptions import (
    AllowlistPackageAlreadyExistsError,
    AllowlistPackageDoesNotExistError,
)

logger = logging.getLogger()


class ConfigHandler:
    """Read certain values into a central ConfigHandler object."""

    def __init__(self, file_path: Optional[str] = None, enforce_file: bool = True):
        self._file_path = file_path or DEFAULT_PROJECT_TOML_FILE
        self._enforce_file = enforce_file
        self._toml = self._get_toml_as_dict()
        self._twyn_data = self._get_twyn_data()

        self.dependency_file: Optional[str] = self._twyn_data.get("dependency_file")
        self.selector_method: Optional[str] = self._twyn_data.get("selector_method")
        self.logging_level: Optional[str] = self._twyn_data.get("logging_level")
        self.allowlist: set[str] = set(self._twyn_data.get("allowlist", []))

    def add_package_to_allowlist(self, package_name: str) -> None:
        if package_name in self.allowlist:
            raise AllowlistPackageAlreadyExistsError(package_name)

        self._create_allowlist_in_toml_if_not_exists()

        self._toml["tool"]["twyn"]["allowlist"].append(package_name)
        self._write_toml()

        logger.warning(f"Package '{package_name}' successfully added to allowlist")

    def remove_package_from_allowlist(self, package_name: str) -> None:
        if package_name not in self.allowlist:
            raise AllowlistPackageDoesNotExistError(package_name)

        self._toml["tool"]["twyn"]["allowlist"].remove(package_name)
        self._write_toml()
        logger.warning(f"Package '{package_name}' successfully removed from allowlist")

    def _get_twyn_data(self) -> dict[str, Any]:
        return self._toml.get("tool", {}).get("twyn", {})

    def _get_toml_file_pointer(self) -> Path:
        """Create a path for the toml file with the format <current working directory>/self.file_path."""
        fp = Path(getcwd()) / Path(self._file_path)

        if not fp.is_file():
            raise FileNotFoundError(f"File not found at path '{fp}'.")

        return fp

    def _write_toml(self) -> None:
        with open(self._get_toml_file_pointer(), "w") as f:
            f.write(dumps(self._toml))

    def _get_toml_as_dict(self) -> dict[str, Any]:
        """Read TOML into a dictionary."""
        try:
            fp = self._get_toml_file_pointer()
        except FileNotFoundError:
            if not self._enforce_file and self._file_path == DEFAULT_PROJECT_TOML_FILE:
                return {}
            raise

        with open(fp, "r") as f:
            content = parse(f.read())
        return parse(dumps(content))

    def _create_allowlist_in_toml_if_not_exists(self) -> None:
        try:
            isinstance(self._toml["tool"]["twyn"]["allowlist"], list)
        except KeyError:
            if "tool" not in self._toml:
                self._toml["tool"] = {}

            if "twyn" not in self._toml["tool"]:
                self._toml["tool"]["twyn"] = {}

            if "allowlist" not in self._toml["tool"]["twyn"]:
                self._toml["tool"]["twyn"]["allowlist"] = []



from __future__ import annotations

import logging
from abc import ABC, abstractmethod
from typing import TYPE_CHECKING, Iterable

from twyn.trusted_packages.constants import ADJACENCY_MATRIX
from twyn.trusted_packages.exceptions import CharacterNotInMatrixError

if TYPE_CHECKING:
    from twyn.trusted_packages.trusted_packages import _PackageNames

logger = logging.getLogger()


class AbstractSelector(ABC):
    @abstractmethod
    def select_similar_names(self, names: _PackageNames, name: str) -> Iterable[str]:
        """Override this to select names that are similar to the provided one."""

    def __str__(self):
        return self.__class__.__name__


class FirstLetterNearbyInKeyboard(AbstractSelector):
    """Selects names that start with a letter that is nearby in an English Keyboard."""

    def select_similar_names(self, names: _PackageNames, name: str) -> Iterable[str]:
        candidate_characters = self._get_candidate_characters(name[0])
        for letter in candidate_characters:
            for candidate in names.get(letter, []):
                yield candidate

    @staticmethod
    def _get_candidate_characters(character: str) -> list[str]:
        if character not in ADJACENCY_MATRIX:
            raise CharacterNotInMatrixError(f"Character '{character}' not supported")

        return ADJACENCY_MATRIX[character] + [character]


class FirstLetterExact(AbstractSelector):
    """Selects names that share the same first letter."""

    def select_similar_names(self, names: _PackageNames, name: str) -> Iterable[str]:
        for candidate in names[name[0]]:
            yield candidate


class AllSimilar(AbstractSelector):
    """Consider all names to be similar."""

    def select_similar_names(self, names: _PackageNames, name: str) -> Iterable[str]:
        for candidates in names.values():
            for candidate in candidates:
                yield candidate



from collections import defaultdict
from dataclasses import dataclass, field
from typing import Any, Set

from twyn.similarity.algorithm import (
    AbstractSimilarityAlgorithm,
    SimilarityThreshold,
)
from twyn.trusted_packages.selectors import AbstractSelector

_PackageNames = defaultdict[str, set[str]]


@dataclass
class TyposquatCheckResult:
    """Represents the result of analyzing a dependency for a possible typosquat."""

    candidate_dependency: str
    similar_dependencies: list[str] = field(default_factory=list)

    def __bool__(self) -> bool:
        return bool(self.similar_dependencies)

    def add(self, similar_name: str) -> None:
        """Add a similar dependency to this typosquat check result."""
        self.similar_dependencies.append(similar_name)


class TrustedPackages:
    """Representation of packages that can be trusted."""

    def __init__(
        self,
        names: Set[str],
        algorithm: AbstractSimilarityAlgorithm,
        selector: AbstractSelector,
        threshold_class: type[SimilarityThreshold],
    ) -> None:
        self.names: _PackageNames = self._create_names_dictionary(names)
        self.threshold_class = threshold_class
        self.selector = selector
        self.algorithm = algorithm

    def __eq__(self, obj: Any) -> bool:
        return isinstance(obj, self.__class__) and self.names == obj.names

    def __contains__(self, obj: Any) -> bool:
        if isinstance(obj, str):
            return obj in self.names[obj[0]]
        return False

    @staticmethod
    def _create_names_dictionary(names: set[str]) -> _PackageNames:
        """Create a dictionary which will group all packages that start with the same letter under the same key."""
        first_letter_names = defaultdict(set)
        for name in names:
            first_letter_names[name[0]].add(name)
        return first_letter_names

    def get_typosquat(
        self,
        package_name: str,
    ) -> TyposquatCheckResult:
        """Check if a given package name is similar to any trusted package and returns it.

        Only if there is a match on the first letter can a package name be
        considered similar to another one. The algorithm provided and the threshold
        are used to determine if the package name can be considered similar.
        """
        threshold = self.threshold_class.from_name(package_name)
        typosquat_result = TyposquatCheckResult(package_name)
        for trusted_package_name in self.selector.select_similar_names(
            names=self.names, name=package_name
        ):
            distance = self.algorithm.get_distance(package_name, trusted_package_name)
            if threshold.is_inside_threshold(distance):
                typosquat_result.add(trusted_package_name)
        return typosquat_result



"""Parser for poetry.lock dependencies."""

import tomllib
from dparse import filetypes

from twyn.dependency_parser.abstract_parser import AbstractParser


class PoetryLockParser(AbstractParser):
    """Parser for poetry.lock."""

    def __init__(self, file_path: str = filetypes.poetry_lock) -> None:
        super().__init__(file_path)

    def parse(self) -> set[str]:
        """Parse poetry.lock dependencies into set of dependency names."""
        data = tomllib.loads(self._read())
        return {dependency["name"] for dependency in data["package"]}



from __future__ import annotations

import enum
from typing import TYPE_CHECKING

import twyn.dependency_parser as dependency_parser
from twyn.trusted_packages import selectors

if TYPE_CHECKING:
    from twyn.dependency_parser.abstract_parser import AbstractParser

SELECTOR_METHOD_MAPPING: dict[str, type[selectors.AbstractSelector]] = {
    "first-letter": selectors.FirstLetterExact,
    "nearby-letter": selectors.FirstLetterNearbyInKeyboard,
    "all": selectors.AllSimilar,
}

DEPENDENCY_FILE_MAPPING: dict[str, type[AbstractParser]] = {
    "requirements.txt": dependency_parser.requirements_txt.RequirementsTxtParser,
    "poetry.lock": dependency_parser.poetry_lock.PoetryLockParser,
}

DEFAULT_SELECTOR_METHOD = "all"
DEFAULT_DEPENDENCY_FILE = "requirements.txt"
DEFAULT_PROJECT_TOML_FILE = "pyproject.toml"


class AvailableLoggingLevels(enum.Enum):
    none = "NONE"
    debug = "DEBUG"
    info = "INFO"
    warning = "WARNING"



import logging
from abc import ABC, abstractmethod
from typing import Any

import requests

from twyn.trusted_packages.constants import TOP_PYPI_PACKAGES, Url
from twyn.trusted_packages.exceptions import (
    EmptyPackagesListError,
    InvalidJSONError,
    InvalidPyPiFormatError,
)

logger = logging.getLogger()


class AbstractPackageReference(ABC):
    """Represents a reference to retrieve trusted packages from."""

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)

    @abstractmethod
    def get_packages(self) -> set[str]:
        """Return the names of the trusted packages available in the reference."""


class TopPyPiReference(AbstractPackageReference):
    """Top PyPi packages retrieved from an online source."""

    def __init__(
        self, source: Url = TOP_PYPI_PACKAGES, *args: Any, **kwargs: Any
    ) -> None:
        super().__init__(*args, **kwargs)
        self.source = source

    def get_packages(self) -> set[str]:
        """Download and parse online source of top Python Package Index packages."""
        packages_info = self._download()
        return self._parse(packages_info)

    def _download(self) -> dict[str, Any]:
        packages = requests.get(self.source)
        packages.raise_for_status()
        try:
            packages_json: dict[str, Any] = packages.json()
        except requests.exceptions.JSONDecodeError as err:
            raise InvalidJSONError from err

        logger.debug(
            f"Successfully downloaded trusted packages list from {self.source}"
        )

        return packages_json

    @staticmethod
    def _parse(packages_info: dict[str, Any]) -> set[str]:
        try:
            names = {row["project"] for row in packages_info["rows"]}
        except KeyError as err:
            raise InvalidPyPiFormatError from err

        if not names:
            raise EmptyPackagesListError

        logger.debug("Successfully parsed trusted packages list")

        return names



from twyn.base.exceptions import TwynError


class DistanceAlgorithmError(TwynError):
    message = "Exception raised while running distance algorithm"


class ThresholdError(TwynError, ValueError):
    message = "Minimum threshold cannot be greater than maximum threshold."



"""Parser for requirements.txt dependencies."""

from dparse import filetypes, parse
from dparse.dependencies import Dependency, DependencyFile

from twyn.dependency_parser.abstract_parser import AbstractParser


class RequirementsTxtParser(AbstractParser):
    """Parser for requirements.txt dependencies."""

    def __init__(self, file_path: str = filetypes.requirements_txt) -> None:
        super().__init__(file_path)

    def parse(self) -> set[str]:
        """Parse requirements.txt dependencies into set of dependency names."""
        dependency_file: DependencyFile = parse(
            self._read(), file_type=filetypes.requirements_txt
        )
        dependencies: list[Dependency] = dependency_file.resolved_dependencies
        return {dependency.name for dependency in dependencies}



"""Dependency parsers."""

from twyn.dependency_parser.poetry_lock import PoetryLockParser
from twyn.dependency_parser.requirements_txt import RequirementsTxtParser

__all__ = ["RequirementsTxtParser", "PoetryLockParser"]



import logging
from typing import Optional

from twyn.base.constants import DEPENDENCY_FILE_MAPPING
from twyn.dependency_parser.abstract_parser import AbstractParser
from twyn.dependency_parser.exceptions import (
    MultipleParsersError,
    NoMatchingParserError,
)

logger = logging.getLogger()


class DependencySelector:
    def __init__(self, dependency_file: Optional[str] = None) -> None:
        self.dependency_file = dependency_file or ""

    @staticmethod
    def _raise_for_selected_parsers(parsers) -> None:
        if len(parsers) > 1:
            raise MultipleParsersError

        if not parsers:
            raise NoMatchingParserError

    def auto_detect_dependency_file_parser(self) -> type[AbstractParser]:
        parsers = [
            dependency_parser
            for dependency_parser in DEPENDENCY_FILE_MAPPING.values()
            if dependency_parser().file_exists()
        ]
        self._raise_for_selected_parsers(parsers)
        logger.debug("Dependencies file found")
        return parsers[0]

    def get_dependency_file_parser_from_file_name(
        self,
    ) -> type[AbstractParser]:
        parsers = []
        for known_dependency_file_name in DEPENDENCY_FILE_MAPPING:
            if self.dependency_file.endswith(known_dependency_file_name):
                parsers.append(DEPENDENCY_FILE_MAPPING[known_dependency_file_name])

        self._raise_for_selected_parsers(parsers)
        return parsers[0]

    def get_dependency_parser(self) -> AbstractParser:
        logger.debug(f"Dependency file: {self.dependency_file}")

        if self.dependency_file:
            logger.debug("Dependency file provided. Assigning a parser.")
            dependency_file_parser = self.get_dependency_file_parser_from_file_name()
        else:
            logger.debug("No dependency file provided. Attempting to locate one.")
            dependency_file_parser = self.auto_detect_dependency_file_parser()

        file_parser = dependency_file_parser()
        logger.debug(f"Assigned {file_parser} parser for local dependencies file.")
        file_parser.raise_for_valid_file()

        return file_parser



import logging
import os
from abc import ABC, abstractmethod
from pathlib import Path

from twyn.base.exceptions import TwynError
from twyn.dependency_parser.exceptions import (
    PathIsNotFileError,
    PathNotFoundError,
)

logger = logging.getLogger()


class AbstractParser(ABC):
    def __init__(self, file_path: str = "") -> None:
        self.file_path = Path(os.path.abspath(os.path.join(os.getcwd(), file_path)))

    def __str__(self):
        return self.__class__.__name__

    def _read(self) -> str:
        content = self.file_path.read_text()
        logger.debug("Successfully read content from local dependencies file")

        return content

    def file_exists(self) -> bool:
        try:
            self.raise_for_valid_file()
        except TwynError:
            return False
        return True

    def raise_for_valid_file(self) -> None:
        if not self.file_path.exists():
            raise PathNotFoundError

        if not self.file_path.is_file():
            raise PathIsNotFileError

    @abstractmethod
    def parse(self) -> set[str]:
        """
        Parse text into dependencies set.

        Parse the file's contents into a set of dependency names (type: str).
        All data other than the dependency names (e.g. whether a dependency is
        a dev dependency or main dependency; version constraints) is omitted.
        """



from __future__ import annotations

import logging
from abc import ABC, abstractmethod

from rapidfuzz.distance import DamerauLevenshtein

from twyn.similarity.exceptions import DistanceAlgorithmError, ThresholdError

logger = logging.getLogger()


class SimilarityThreshold:
    LENGTH_CUTOFF = 5
    MIN_VALUE = 1.0
    MAX_FOR_SHORT_WORDS = 1.0
    MAX_FOR_LONG_WORDS = 2.0

    def __init__(self, max: float) -> None:
        self.min = self.MIN_VALUE
        self.max = max

        if self.min > self.max:
            raise ThresholdError

    @classmethod
    def from_name(cls, name: str) -> SimilarityThreshold:
        name_length = len(name)
        if name_length <= cls.LENGTH_CUTOFF:
            logger.debug(f"max length of {cls.MAX_FOR_SHORT_WORDS} selected for {name}")
            return cls(max=cls.MAX_FOR_SHORT_WORDS)
        logger.debug(f"max length of {cls.MAX_FOR_LONG_WORDS} selected for {name}")
        return cls(
            max=cls.MAX_FOR_LONG_WORDS
        )  # we allow more typos if the name is longer

    def is_inside_threshold(self, value: float) -> bool:
        return self.min <= value <= self.max


class AbstractSimilarityAlgorithm(ABC):
    """Algorithm that can compare sequences based of a particular similarity measure."""

    def get_distance(self, first_sequence: str, second_sequence: str) -> float | int:
        """
        Perform the alignment between sequences and return the computed distance.

        Will raise DistanceAlgorithmError if an exception occurs.
        """
        try:
            return self._run_algorithm(first_sequence, second_sequence)
        except Exception as exc:
            raise DistanceAlgorithmError from exc

    @abstractmethod
    def _run_algorithm(self, first_sequence: str, second_sequence: str) -> float | int:
        """Abstract method that runs the selected algorithm for computing the distance between two words."""


class EditDistance(AbstractSimilarityAlgorithm):
    """Levenshtein algorithm that computes the edit distance between words."""

    def _run_algorithm(self, first_sequence: str, second_sequence: str) -> int:
        return DamerauLevenshtein.distance(s1=first_sequence, s2=second_sequence)



from twyn.base.exceptions import TwynError


class AllowlistError(TwynError):
    def __init__(self, package_name: str = ""):
        message = self.message.format(package_name) if package_name else self.message
        super().__init__(message)


class AllowlistPackageAlreadyExistsError(AllowlistError):
    message = "Package '{}' is already present in the allowlist. Skipping."


class AllowlistPackageDoesNotExistError(AllowlistError):
    message = "Package '{}' is not present in the allowlist. Skipping."



import logging
import re
from typing import Optional

from rich.logging import RichHandler
from rich.progress import track

from twyn.base.constants import (
    DEFAULT_SELECTOR_METHOD,
    SELECTOR_METHOD_MAPPING,
    AvailableLoggingLevels,
)
from twyn.core.config_handler import ConfigHandler
from twyn.dependency_parser.dependency_selector import DependencySelector
from twyn.similarity.algorithm import EditDistance, SimilarityThreshold
from twyn.trusted_packages import TopPyPiReference
from twyn.trusted_packages.selectors import AbstractSelector
from twyn.trusted_packages.trusted_packages import (
    TrustedPackages,
    TyposquatCheckResult,
)

logging.basicConfig(
    format="%(message)s",
    datefmt="[%X]",
    handlers=[RichHandler(rich_tracebacks=True, show_path=False)],
)
logger = logging.getLogger()


def check_dependencies(
    config_file: Optional[str],
    dependency_file: str,
    selector_method: str,
    verbosity: AvailableLoggingLevels = AvailableLoggingLevels.none,
) -> bool:
    """Check if dependencies could be typosquats."""
    config = get_configuration(config_file, dependency_file, selector_method, verbosity)
    trusted_packages = TrustedPackages(
        names=TopPyPiReference().get_packages(),
        algorithm=EditDistance(),
        selector=get_candidate_selector(config.selector_method),
        threshold_class=SimilarityThreshold,
    )
    normalized_allowlist_packages = normalize_packages(config.allowlist)
    dependencies = get_parsed_dependencies(config.dependency_file)
    normalized_dependencies = normalize_packages(dependencies)

    errors: list[TyposquatCheckResult] = []
    for dependency in track(normalized_dependencies, description="Processing..."):
        if dependency in normalized_allowlist_packages:
            logger.info(f"Dependency {dependency} is in the allowlist")
            continue

        logger.info(f"Analyzing {dependency}")
        if dependency not in trusted_packages and (
            typosquat_results := trusted_packages.get_typosquat(dependency)
        ):
            errors.append(typosquat_results)

    for possible_typosquats in errors:
        logger.error(
            f"Possible typosquat detected: `{possible_typosquats.candidate_dependency}`, "
            f"did you mean any of [{', '.join(possible_typosquats.similar_dependencies)}]?"
        )

    return bool(errors)


def get_configuration(
    config_file: Optional[str],
    dependency_file: str,
    selector_method: str,
    verbosity: AvailableLoggingLevels,
) -> ConfigHandler:
    """Read configuration and return configuration object.

    Selects the appropriate values based on priorities between those in the file, and those directly provided.
    """
    # Read config from file
    config = ConfigHandler(file_path=config_file, enforce_file=False)

    # Set logging level according to priority order
    logging_level: AvailableLoggingLevels = get_logging_level(
        logging_level=verbosity,
        config_logging_level=config.logging_level,
    )
    set_logging_level(logging_level)
    config.logging_level = logging_level.value

    # Set selector method according to priority order
    config.selector_method = (
        selector_method or config.selector_method or DEFAULT_SELECTOR_METHOD
    )

    # Set dependency file according to priority order
    config.dependency_file = dependency_file or config.dependency_file or None
    return config


def get_logging_level(
    logging_level: AvailableLoggingLevels,
    config_logging_level: Optional[str],
) -> AvailableLoggingLevels:
    """Return the appropriate logging level, considering that the one in config has less priority than the one passed directly."""
    if logging_level is AvailableLoggingLevels.none:
        if config_logging_level:
            return AvailableLoggingLevels[config_logging_level.lower()]
        else:
            # default logging level
            return AvailableLoggingLevels.warning

    return logging_level


def set_logging_level(logging_level: AvailableLoggingLevels) -> None:
    logger.setLevel(logging_level.value)
    logger.debug(f"Logging level: {logging_level.value}")


def get_candidate_selector(selector_method_name: Optional[str]) -> AbstractSelector:
    logger.debug(f"Selector method received {selector_method_name}")
    selector_method_name = selector_method_name or DEFAULT_SELECTOR_METHOD
    selector_method = SELECTOR_METHOD_MAPPING[selector_method_name]()
    logger.debug(f"Instantiated {selector_method} selector")
    return selector_method


def get_parsed_dependencies(dependency_file: Optional[str] = None) -> set[str]:
    dependency_parser = DependencySelector(dependency_file).get_dependency_parser()
    dependencies = dependency_parser.parse()
    logger.debug("Successfully parsed local dependencies file.")
    return dependencies


def normalize_packages(packages: set[str]) -> set[str]:
    """Normalize dependency names according to PyPi https://packaging.python.org/en/latest/specifications/name-normalization/."""
    return {re.sub(r"[-_.]+", "-", name).lower() for name in packages}



from twyn.base.exceptions import TwynError


class PathIsNotFileError(TwynError):
    message = "Specified dependencies path is not a file"


class PathNotFoundError(TwynError, FileNotFoundError):
    message = "Specified dependencies file path does not exist"


class NoMatchingParserError(TwynError):
    message = "Could not assign a dependency file parser. Please specify it with --dependency-file"


class MultipleParsersError(TwynError):
    message = (
        "Can't auto detect dependencies file to parse. More than one format was found."
    )
